﻿#include "Obelisk.h"

#include "CGameController.h"
#include "CGameState.h"
#include "COverworld.h"
#include "CIsland.h"

using namespace Obelisk;



COverworld::COverworld(IObelisk* aObelisk, CGameController* aOwner)
{
	mObelisk = aObelisk;
	mOwner = aOwner;

	// top-down perspective projection matrix
	const float Zn = -2, Zf = -1024;
	float a = Zf / (Zn - Zf);
	memset(&mProjectionMatrix, 0, 16 * sizeof(float));
	mProjectionMatrix[0 * 4 + 0] = 2 / (1024.0f / 100);
	mProjectionMatrix[1 * 4 + 1] = -2 * (1 / 0.75f) / (1024.0f / 100);
	mProjectionMatrix[2 * 4 + 2] = a;
	mProjectionMatrix[3 * 4 + 2] = -a * Zn;
	mProjectionMatrix[2 * 4 + 3] = -1;

	mCameraX = 20;
	mCameraY = 20;
	mCameraZ = 50;
	mCursor = false;

	mSelectedIsland = NULL;

	mBoatBob = 0;

	mFirstInRange = true;
}



COverworld::~COverworld()
{
}



void COverworld::Update()
{
	int CursorX;
	int CursorY;
	mObelisk->GetCursorPos(CursorX, CursorY);

	if (mSelectedIsland == NULL)
	{
		double ScreenXSize = ((double)(mCameraZ - 2) / (100 - 2) * (512 - 1.5) + 1.5) * 2;
		double ScreenYSize = ((double)(mCameraZ - 2) / (100 - 2) * (384 - 1.125) + 1.125) * 2;
		// mouse in game-coordinates
		float MouseX = (float)((double)(CursorX - 512) / 1023 * ScreenXSize) + mCameraX;
		float MouseY = (float)((double)(CursorY - 384) / 767 * ScreenYSize) + mCameraY;

		CIsland *CurrentIsland = NULL;
		CCollection<CIsland> Islands = mOwner->Game()->GetIslands();
		foreach (Islands)
		{
			if (pow(Islands->GetX() - MouseX, 2) + pow(Islands->GetY() - MouseY, 2) <
					5 * 5)
			{
				CurrentIsland = Islands.Current();
				break;
			}
		}
		mCursor = CurrentIsland != NULL;

		if (mObelisk->MouseButtonClicked(false))
		{
			if (CurrentIsland != NULL)
				mSelectedIsland = CurrentIsland;
			else
				mOwner->Game()->MoveBoat(MouseX, MouseY);
		}
		if (mObelisk->IsKeyDown(kA) && mCameraZ > 20)
			mCameraZ--;
		if (mObelisk->IsKeyDown(kZ) && mCameraZ < 100)
			mCameraZ++;
	}
	else
	{
		mCursor = false;

		mIslandInRange = pow(mSelectedIsland->GetX() - mOwner->Game()->GetBoatX(), 2) +
				pow(mSelectedIsland->GetY() - mOwner->Game()->GetBoatY(), 2) < 12 * 12;
		if (mIslandInRange && mFirstInRange)
		{
			mFirstInRange = false;
			mOwner->FirstInRange();
		}

		if (mObelisk->MouseButtonClicked(false))
		{
			if (CursorX >= 682 && CursorX < 682 + 30 && CursorY >= 134 && CursorY < 134 + 30)
			{
				mSelectedIsland = NULL;
			}
			else if (CursorX >= -80+512 && CursorY >= 384+130 && CursorX < 512+80 && CursorY < 384+160)
			{
				if (mIslandInRange)
				{
					if (mSelectedIsland->GetOwner() == 1)
					{
						mOwner->Combat(mSelectedIsland);
						mSelectedIsland = NULL;
					}
					else
						mOwner->Game()->Refuel();
				}
				else
				{
					float Angle = atan2(mSelectedIsland->GetY() - mOwner->Game()->GetBoatY(),
							mSelectedIsland->GetX() - mOwner->Game()->GetBoatX());
					float Distance = sqrt(pow(mSelectedIsland->GetX() - mOwner->Game()->GetBoatX(),
							2) + pow(mSelectedIsland->GetY() - mOwner->Game()->GetBoatY(), 2));
					mOwner->Game()->MoveBoat(mOwner->Game()->GetBoatX() + cos(Angle) * (Distance -
							10), mOwner->Game()->GetBoatY() + sin(Angle) * (Distance - 10));
				}
			}
		}
	}

	mBoatBob = (mBoatBob + 1) % 70;
	mCameraX = mOwner->Game()->GetBoatX();
	mCameraY = mOwner->Game()->GetBoatY();
}



void COverworld::Render()
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glScalef(1, -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	CVector BackV0(-1, -1, 0);
	CVector BackV1(1, -1, 0);
	CVector BackV2(1, 1, 0);
	CVector BackV3(-1, 1, 0);
	mObelisk->TriColour(BackV0, BackV1, BackV2, CColour(0.1f, 0.2f, 0.85f));
	mObelisk->TriColour(BackV0, BackV2, BackV3, CColour(0.1f, 0.2f, 0.85f));

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glMultMatrixf(mProjectionMatrix);
	glTranslatef(-mCameraX, -mCameraY, -mCameraZ);

	const int cIslandSize = 3;
	CCollection<CIsland> Islands = mOwner->Game()->GetIslands();
	foreach (Islands)
	{
		glPushMatrix();
		glTranslatef((float)Islands->GetX(), (float)Islands->GetY(), 0);
		CVector V0((float)-cIslandSize, (float)-cIslandSize, 0);
		CVector V1((float)cIslandSize, (float)-cIslandSize, 0);
		CVector V2((float)-cIslandSize, (float)cIslandSize, 0);
		CVector V3((float)cIslandSize, (float)cIslandSize, 0);
		CColour Colour = Islands->GetOwner() == 0 ? CColour(0, 0.9f, 0) : CColour(0.9f, 0, 0);
		mObelisk->TriColour(V0, V1, V2, Colour);
		mObelisk->TriColour(V1, V3, V2, Colour);
		glPopMatrix();
	}

	glPushMatrix();
	mObelisk->SetLinearFiltering(true);
	float BoatOffset = (float)sin((float)mBoatBob / 70 * M_PI * 2) * 1;
	glTranslatef(mOwner->Game()->GetBoatX(), mOwner->Game()->GetBoatY() + BoatOffset, 0);
	const float cBoatXSize = 0.5f * 15;
	const float cBoatYSize = 117.0f / 140 / 2 * 15;
	CVector BoatV0(-cBoatXSize, -cBoatYSize, 0);
	CVector BoatV1(cBoatXSize, -cBoatYSize, 0);
	CVector BoatV2(cBoatXSize, cBoatYSize, 0);
	CVector BoatV3(-cBoatXSize, cBoatYSize, 0);
	CTexture BoatTexture = mObelisk->GetTexture(mOwner->Game()->GetBoatDir() == 0 ?
			SL("BoatLeft.bmp") : SL("BoatRight.bmp"));
	mObelisk->TriTexture(BoatV0, 0.0f, 0.0f, BoatV1, 1.0f, 0.0f, BoatV2, 1.0f, 1.0f,
			BoatTexture);
	mObelisk->TriTexture(BoatV0, 0.0f, 0.0f, BoatV2, 1.0f, 1.0f, BoatV3, 0.0f, 1.0f,
			BoatTexture);
	mObelisk->SetLinearFiltering(false);
	glPopMatrix();

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	mObelisk->SetSpriteViewport(1024, 768);

	if (mSelectedIsland != NULL)
	{
		glPushMatrix();
		glTranslatef(512, 384, 0);

		mObelisk->SetAlpha(0.7f);
		mOwner->DrawRect(-200, -250, 200, 250, CColour(0.3f, 0.3f, 0.3f));
		mObelisk->SetAlpha(1.0f);

		mOwner->DrawRect(-80, 130, 80, 160, CColour(0.7f, 0.7f, 0.7f));
		if (mIslandInRange)
		{
			if (mSelectedIsland->GetOwner() == 1)
				mObelisk->DrawText(SL("Attack"), -50, 130);
			else
				mObelisk->DrawText(SL("Refuel"), -50, 130);
		}
		else
		{
			mOwner->DrawRect(-80, 130, 80, 160, CColour(0.7f, 0.7f, 0.7f));
			mObelisk->DrawText(SL("Move To"), -59, 130);
		}

		mObelisk->DrawText(SL("X"), 175, -250);
		mObelisk->DrawText(SL("Oil rating: ") + mObelisk->IntToString(mSelectedIsland->GetOilRate()),
				-180, -200);
		if (mSelectedIsland->GetOwner() == 1)
		{
			mObelisk->DrawText(SL("Level: ") + mObelisk->IntToString(mSelectedIsland->GetLevel()),
					-180, -170);
		}

		glPopMatrix();
	}

	mObelisk->DrawText(SL("Fuel: ") + mObelisk->IntToString(mOwner->Game()->GetBoatFuel() /
			10), 10, 0);
	mObelisk->DrawText(SL("Rank: ") + mObelisk->IntToString(mOwner->Game()->GetRank()) +
			SL(" (") + mObelisk->IntToString(mOwner->Game()->GetRankOil()) + SL("/") +
			mObelisk->IntToString(mOwner->Game()->GetRankOilMax()) + SL(")"), 10, 30);
	mObelisk->DrawText(SL("Helicopters: ") + mObelisk->IntToString(mOwner->Game()->GetNumHelis()) +
			SL(" (") + mObelisk->IntToString(mOwner->Game()->GetHeliOil()) + SL("/") +
			mObelisk->IntToString(mOwner->Game()->GetHeliOilMax()) + SL(")"), 10, 60);
	mObelisk->DrawText(SL("Islands: ") + mObelisk->IntToString(mOwner->Game()->GetNumYourIslands()),
			10, 90);
	mObelisk->DrawText(SL("Oil: ") + mObelisk->IntToString(mOwner->Game()->GetOil()),
			10, 120);

	mObelisk->DrawText(SL("A - Zoom in"), 10, 700);
	mObelisk->DrawText(SL("Z - Zoom out"), 10, 730);

	if (!mOwner->IsCursorShowing())
	{
		int CursorX;
		int CursorY;
		mObelisk->GetCursorPos(CursorX, CursorY);
		if (CursorX >= 0 && CursorX < 1024 && CursorY >= 0 && CursorY < 768)
			mObelisk->DrawSprite(mObelisk->GetTexture(mCursor ? SL("Select.bmp") :
					SL("Crosshair.bmp")), CursorX, CursorY, -1, -1, 0.5, 0.5);
	}
}