﻿#include "Obelisk.h"

#include "CGameController.h"
#include "CScriptEngine.h"
#include "IScript.h"
#include "CScriptStartCutscene.h"
#include "CScriptAttackHelp.h"
#include "CScriptCombatHelp.h"
#include "CScriptRankHelp.h"
#include "CScriptCutscene.h"
#include "CScriptFinalCutscene.h"

using namespace Obelisk;



CScriptEngine::CScriptEngine(IObelisk* aObelisk, CGameController* aOwner)
{
	mObelisk = aObelisk;
	mOwner = aOwner;
	mScript = NULL;
}



CScriptEngine::~CScriptEngine()
{
	if (mScript != NULL)
		delete mScript;
}



bool CScriptEngine::Update()
{
	if (mScript == NULL)
		return false;
	bool Result;
	if (mScript->Update(Result, mObelisk, this))
	{
		delete mScript;
		mScript = NULL;
	}
	return Result;
}



void CScriptEngine::Render()
{
	if (mScript != NULL)
	{
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		mObelisk->SetSpriteViewport(1024, 768);
		mScript->Render(mObelisk, this);
	}
}



bool CScriptEngine::IsEnterPressed()
{
	return mOwner->IsEnterPressed();
}



bool CScriptEngine::IsEscapePressed()
{
	return mOwner->IsEscapePressed();
}



void CScriptEngine::ShowProfileCut(EScriptProfile aProfile, int aPos)
{
	mObelisk->SetAlpha(0.85f);
	mOwner->DrawRect(0, 0, 1024, 768, CColour(0.1f, 0.1f, 0.1f));
	mObelisk->SetAlpha(1.0f);

	CTexture Texture;
	if (aProfile == pCmdr)
		Texture = mObelisk->GetTexture(SL("Cmdr.bmp"));
	else if (aProfile == pYou)
		Texture = mObelisk->GetTexture(SL("You.bmp"));
	else if (aProfile == pBadGuy)
		Texture = mObelisk->GetTexture(SL("BadGuy.bmp"));
	mObelisk->DrawSprite(Texture, aPos == 0 ? 20 : (1024 - 220), 200);
	mObelisk->DrawText(SL("(Click or Enter)"), 512 - 233 / 2, 450);
}



void CScriptEngine::ShowTextCut(int aPos, int aLine, CString aText)
{
	mObelisk->DrawText(aText, aPos == 0 ? 250 : 150, aLine * 40 + 240);
}



void CScriptEngine::ShowProfileInner(EScriptProfile aProfile)
{
	CTexture Texture;
	if (aProfile == pCmdr)
		Texture = mObelisk->GetTexture(SL("Cmdr.bmp"));
	else if (aProfile == pYou)
		Texture = mObelisk->GetTexture(SL("You.bmp"));
	else if (aProfile == pBadGuy)
		Texture = mObelisk->GetTexture(SL("BadGuy.bmp"));
	mObelisk->DrawSprite(Texture, 10, 100);
}



void CScriptEngine::ShowTextInner(int aLine, CString aText)
{
	mObelisk->DrawText(aText, 220, aLine * 30 + 140);
}



void CScriptEngine::DrawRect(float aX0, float aY0, float aX1, float aY1, const CColour& aColour)
{
	mOwner->DrawRect(aX0, aY0, aX1, aY1, aColour);
}



void CScriptEngine::Win()
{
	mOwner->WIN();
}



void CScriptEngine::GameStart()
{
	mScript = new CScriptStartCutscene();
}



void CScriptEngine::FirstInRange()
{
	if (mScript != NULL)
		delete mScript;
	mScript = new CScriptAttackHelp();
}



void CScriptEngine::FirstCombat()
{
	if (mScript != NULL)
		delete mScript;
	mScript = new CScriptCombatHelp();
}



void CScriptEngine::FirstRank()
{
	if (mScript != NULL)
		delete mScript;
	mScript = new CScriptRankHelp();
}



void CScriptEngine::FirstIslandWon()
{
	if (mScript != NULL)
		delete mScript;
	mScript = new CScriptCutscene(0);
}



void CScriptEngine::LevelWon(int aLevel)
{
	IScript *NewScript = NULL;
	if (aLevel == 3)
		NewScript = new CScriptCutscene(1);
	if (aLevel == 5)
		NewScript = new CScriptCutscene(2);
	if (aLevel == 7)
		NewScript = new CScriptCutscene(3);
	if (aLevel == 9)
		NewScript = new CScriptCutscene(4);
	if (aLevel >= 10)
		NewScript = new CScriptFinalCutscene();
	if (NewScript == NULL)
		return;
	if (mScript != NULL)
		delete mScript;
	mScript = NewScript;
}